package com.onsiteservice.common.service;

import com.alibaba.fastjson.JSONObject;
import com.onsiteservice.common.redis.RedisUtils;
import com.onsiteservice.constant.enums.ServiceUserTypeEnum;
import com.onsiteservice.dao.mapper.user.UserMapper;
import com.onsiteservice.entity.user.User;
import com.onsiteservice.util.aliyun.SmsUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Condition;

import javax.annotation.Resource;
import java.util.Map;

/**
 * 业务短信服务
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class CommonSmsService {

    @Value("${spring.profiles.active}")
    private String env;

    @Value("${aliyun.sms.template-code.change-phone}")
    private String changePhoneTmpCode; // 手机号换绑

    @Value("${aliyun.sms.template-code.payable}")
    private String payableTmpCode; // 通知用户完成估价请支付

    @Value("${aliyun.sms.template-code.dispatched}")
    private String dispatchedTmpCode; // 通知用户已经派单

    @Value("${aliyun.sms.template-code.redispatched}")
    private String redispatchedTmpCode; // 通知用户已经重新派单

    @Value("${aliyun.sms.template-code.paid}")
    private String paidTmpCode; // 手机号换绑

    @Value("${aliyun.sms.template-code.valuator}")
    private String valuatorTmpCode; // 通知客户已派估价员

    @Value("${aliyun.sms.template-code.revaluator}")
    private String revaluatorTmpCode; // 通知客户已重新指派估价员

    @Value("${aliyun.sms.template-code.submit}")
    private String submitOrderTmpCode;

    @Value("${aliyun.sms.sign}")
    private String sign;

    @Resource
    private SmsUtils smsUtils;
    @Resource
    private RedisUtils redisUtil;

    @Resource
    private UserMapper userMapper;

    /***
     * 禁用短信方法
     */
    private boolean disableSms() {
//        return "dev".equals(env);
        return false;
    }


    /***
     * 发送改变信息验证码
     */
    public void sendChangePhone(String code, String phone) {
        if (StringUtils.isEmpty(phone)) {
            return;
        }
        try {
            smsUtils.send(sign, changePhoneTmpCode, JSONObject.toJSONString(Map.of("code", code)), phone);
        } catch (Exception e) {
            log.error("发送短信错误", e);
        }
    }

    public void payable(String phone, String servicePhone, String serviceName) {
        if (StringUtils.isEmpty(phone)) {
            return;
        }
        try {
            smsUtils.send(sign, payableTmpCode, JSONObject.toJSONString(Map.of("phone", servicePhone, "name", serviceName)), phone);
        } catch (Exception e) {
            log.error("发送短信错误", e);
        }
    }

    public void dispatched(String phone, String servicePhone, String serviceName) {
        if (StringUtils.isEmpty(phone)) {
            return;
        }
        try {
            smsUtils.send(sign, dispatchedTmpCode, JSONObject.toJSONString(Map.of("phone", servicePhone, "name", serviceName)), phone);
        } catch (Exception e) {
            log.error("发送短信错误", e);
        }
    }
    public void redispatched(String phone, String servicePhone, String serviceName) {
        if (StringUtils.isEmpty(phone)) {
            return;
        }
        try {
            smsUtils.send(sign, redispatchedTmpCode, JSONObject.toJSONString(Map.of("phone", servicePhone, "name", serviceName)), phone);
        } catch (Exception e) {
            log.error("发送短信错误", e);
        }
    }

    public void paid(String phone) {
        if (StringUtils.isEmpty(phone)) {
            return;
        }
        try {
            smsUtils.send(sign, paidTmpCode, null, phone);
        } catch (Exception e) {
            log.error("发送短信错误", e);
        }
    }

    public void valuator(String phone, String servicePhone, String serviceName) {
        if (StringUtils.isEmpty(phone)) {
            return;
        }
        try {
            smsUtils.send(sign, valuatorTmpCode, JSONObject.toJSONString(Map.of("phone", servicePhone, "name", serviceName)), phone);
        } catch (Exception e) {
            log.error("发送短信错误", e);
        }
    }

    public void revaluator(String phone, String servicePhone, String serviceName) {
        if (StringUtils.isEmpty(phone)) {
            return;
        }
        try {
            smsUtils.send(sign, revaluatorTmpCode, JSONObject.toJSONString(Map.of("phone", servicePhone, "name", serviceName)), phone);
        } catch (Exception e) {
            log.error("发送短信错误", e);
        }
    }

    public void submit(String serviceName) {
        Condition userCondition = new Condition(User.class);
        userCondition.createCriteria().andEqualTo("roleType", ServiceUserTypeEnum.ADMIN.getId())
                .andIsNotNull("phone");
        var userList = userMapper.selectByCondition(userCondition);
        if (userList.isEmpty()) {
            return;
        }
        String[] phones = userList.stream().map(User::getPhone).toArray(String[]::new);
        try {
            smsUtils.send(sign, submitOrderTmpCode, JSONObject.toJSONString(Map.of( "name", serviceName)), phones);
        } catch (Exception e) {
            log.error("发送短信错误", e);
        }
    }

}
